@@ -87,6 +87,26 @@ Rerun `rake` under MRI Ruby 1.8.x/1.9.x to cross/native compile.
     end
 
     private
+    # copy other gem files to staging directory
+    def define_staging_file_tasks(files, lib_path, stage_path, platf, ruby_ver)
+      files.each do |gem_file|
+        # ignore directories and the binary extension
+        next if File.directory?(gem_file) || gem_file == "#{lib_path}/#{binary(platf)}"
+        stage_file = "#{stage_path}/#{gem_file}"
+
+        # copy each file from base to stage directory
+        unless Rake::Task.task_defined?(stage_file) then
+          directory File.dirname(stage_file)
+          file stage_file => [File.dirname(stage_file), gem_file] do
+            cp gem_file, stage_file
+          end
+        end
+
+        # append each file to the copy task
+        task "copy:#{@name}:#{platf}:#{ruby_ver}" => [stage_file]
+      end
+    end
+
     def define_compile_tasks(for_platform = nil, ruby_ver = RUBY_VERSION)
       # platform usage
       platf = for_platform || platform
@@ -120,24 +140,7 @@ Rerun `rake` under MRI Ruby 1.8.x/1.9.x to cross/native compile.
       end
 
       # copy other gem files to staging directory
-      if @gem_spec
-        @gem_spec.files.each do |gem_file|
-          # ignore directories and the binary extension
-          next if File.directory?(gem_file) || gem_file == "#{lib_path}/#{binary(platf)}"
-          stage_file = "#{stage_path}/#{gem_file}"
-
-          # copy each file from base to stage directory
-          unless Rake::Task.task_defined?(stage_file) then
-            directory File.dirname(stage_file)
-            file stage_file => [File.dirname(stage_file), gem_file] do
-              cp gem_file, stage_file
-            end
-          end
-
-          # append each file to the copy task
-          task "copy:#{@name}:#{platf}:#{ruby_ver}" => [stage_file]
-        end
-      end
+      define_staging_file_tasks(@gem_spec.files, lib_path, stage_path, platf, ruby_ver) if @gem_spec
 
       # binary in temporary folder depends on makefile and source files
       # tmp/extension_name/extension_name.{so,bundle}
@@ -253,9 +256,7 @@ Java extension should be preferred.
           spec.files += ext_files
 
           # expose gem specification for customization
-          if callback
-            callback.call(spec)
-          end
+          callback.call(spec) if callback
 
           # Generate a package for this gem
           pkg = Gem::PackageTask.new(spec) do |pkg|
@@ -266,6 +267,9 @@ Java extension should be preferred.
             pkg.package_files.clear
           end
 
+          # copy other gem files to staging directory if added by the callback
+          define_staging_file_tasks(spec.files, lib_path, stage_path, platf, ruby_ver)
+
           # Copy from staging directory to gem package directory.
           # This is derived from the code of Gem::PackageTask
           # but uses stage_path as source directory.
@@ -380,6 +384,17 @@ Java extension should be preferred.
       # copy mkmf from cross-ruby location
       file "#{tmp_path}/mkmf.rb" => [mkmf_file] do |t|
         cp t.prerequisites.first, t.name
+        if ruby_ver < "1.9" && "1.9" <= RUBY_VERSION
+          File.open(t.name, 'r+t') do |f|
+            content = f.read
+            content.sub!(/^(      break )\*(defaults)$/, '\\1\\2.first')
+            content.sub!(/^(    return )\*(defaults)$/, '\\1\\2.first')
+            content.sub!(/^(  mfile\.)print( configuration\(srcprefix\))$/, '\\1puts\\2')
+            f.rewind
+            f.write content
+            f.truncate(f.tell)
+          end
+        end
       end
 
       # genearte fake.rb for different ruby versions
@@ -471,6 +486,11 @@ Java extension should be preferred.
 
     def fake_rb(platform, version)
       <<-FAKE_RB
+        # Pre-load resolver library before faking, in order to avoid error
+        # "cannot load such file -- win32/resolv" when it is required later on.
+        # See also: https://github.com/tjschuck/rake-compiler-dev-box/issues/5
+        require 'resolv'
+
         class Object
           remove_const :RUBY_PLATFORM
           remove_const :RUBY_VERSION
