@@ -338,9 +338,39 @@ describe Rake::ExtensionTask do
         }.should_not raise_error
       end
 
+      it 'should generate additional rake tasks if files are added when cross compiling' do
+        config = mock(Hash)
+        config.stub!(:[]).and_return('/rubies/1.9.1/rbconfig.rb')
+        YAML.stub!(:load_file).and_return(config)
+
+        # Use a real spec instead of a mock because define_native_tasks dups and
+        # calls methods on Gem::Specification, which is more than mock can do.
+        spec = Gem::Specification.new do |s|
+          s.name = 'my_gem'
+          s.platform = Gem::Platform::RUBY
+        end
+
+        # Gem::PackageTask calls Rake::PackageTask which sets Gem.configuration.verbose,
+        # which initializes Gem::ConfigFile,
+        # which gets mad if it cannot find `sysconfdir`/gemrc
+        Gem.stub_chain(:configuration, :verbose=).and_return(true)
+
+        ENV['RUBY_CC_VERSION'] = '1.9.1'
+        Rake::ExtensionTask.new('extension_one', spec) do |ext|
+          ext.cross_compile = true
+          ext.cross_platform = 'universal-unknown'
+          ext.cross_compiling do |gem_spec|
+            gem_spec.files << 'somedir/somefile'
+          end
+        end
+        Rake::Task['native:my_gem:universal-unknown'].execute
+        Rake::Task.should have_defined("tmp/universal-unknown/stage/somedir")
+        Rake::Task.should have_defined("tmp/universal-unknown/stage/somedir/somefile")
+      end
+
       it 'should allow usage of RUBY_CC_VERSION to indicate a different version of ruby' do
         config = mock(Hash)
-        config.should_receive(:[]).with("rbconfig-i386-mingw32-1.9.1").and_return('/path/to/ruby/1.9.1/rbconfig.rb')
+        config.should_receive(:[]).with("rbconfig-i386-mingw32-1.9.1").and_return('/rubies/1.9.1/rbconfig.rb')
         YAML.stub!(:load_file).and_return(config)
 
         ENV['RUBY_CC_VERSION'] = '1.9.1'
@@ -351,8 +381,8 @@ describe Rake::ExtensionTask do
 
       it 'should allow multiple versions be supplied to RUBY_CC_VERSION' do
         config = mock(Hash)
-        config.should_receive(:[]).once.with("rbconfig-i386-mingw32-1.8.6").and_return('/path/to/ruby/1.8.6/rbconfig.rb')
-        config.should_receive(:[]).once.with("rbconfig-i386-mingw32-1.9.1").and_return('/path/to/ruby/1.9.1/rbconfig.rb')
+        config.should_receive(:[]).once.with("rbconfig-i386-mingw32-1.8.6").and_return('/rubies/1.8.6/rbconfig.rb')
+        config.should_receive(:[]).once.with("rbconfig-i386-mingw32-1.9.1").and_return('/rubies/1.9.1/rbconfig.rb')
         YAML.stub!(:load_file).and_return(config)
 
         ENV['RUBY_CC_VERSION'] = '1.8.6:1.9.1'
@@ -368,7 +398,7 @@ describe Rake::ExtensionTask do
       context "(cross compile for multiple versions)" do
         before :each do
           config = mock(Hash)
-          config.stub!(:[]).and_return('/path/to/ruby/1.8.6/rbconfig.rb', '/path/to/ruby/1.9.1/rbconfig.rb')
+          config.stub!(:[]).and_return('/rubies/1.8.6/rbconfig.rb', '/rubies/1.9.1/rbconfig.rb')
           YAML.stub!(:load_file).and_return(config)
 
           ENV['RUBY_CC_VERSION'] = '1.8.6:1.9.1'
@@ -475,24 +505,34 @@ describe Rake::ExtensionTask do
   end
 
   def mock_config_yml
-    {
-      'rbconfig-i386-mingw32-1.8.6' => '/some/path/version/1.8/to/rbconfig.rb',
-      'rbconfig-universal-unknown-1.8.6' => '/some/path/version/1.8/to/rbconfig.rb',
-      'rbconfig-i386-mingw32-1.8.7' => '/some/path/version/1.8/to/rbconfig.rb',
-      'rbconfig-universal-known-1.8.7' => '/some/path/version/1.8/to/rbconfig.rb',
-      'rbconfig-universal-unknown-1.8.7' => '/some/path/version/1.8/to/rbconfig.rb',
-      'rbconfig-universal-unknown-1.9.1' => '/some/path/version/1.9.1/to/rbconfig.rb',
-      'rbconfig-universal-unknown-1.9.2' => '/some/path/version/1.9.1/to/rbconfig.rb',
-      'rbconfig-universal-unknown-1.9.3' => '/some/path/version/1.9.1/to/rbconfig.rb',
-      'rbconfig-i386-mingw32-1.9.3' => '/some/path/version/1.9.1/to/rbconfig.rb',
-      'rbconfig-universal-known-1.9.3' => '/some/path/version/1.9.1/to/rbconfig.rb',
-      'rbconfig-universal-known-2.0.0' => '/some/path/version/2.0.0/to/rbconfig.rb',
-      'rbconfig-universal-unknown-1.9.3' => '/some/path/version/1.9.1/to/rbconfig.rb',
-      'rbconfig-universal-unknown-2.0.0' => '/some/path/version/2.0.0/to/rbconfig.rb',
-      'rbconfig-i386-mingw32-2.0.0' => '/some/path/version/2.0.0/to/rbconfig.rb',
-      'rbconfig-x64-mingw32-2.0.0' => '/some/path/version/2.0.0/to/rbconfig.rb',
-      'rbconfig-x64-mingw32-3.0.0' => '/some/fake/version/3.0.0/to/rbconfig.rb'
+    return @mock_config_yml if @mock_config_yml
+
+    versions = {
+      "1.8.6"      => "1.8",
+      "1.8.7"      => "1.8",
+      "1.9.3"      => "1.9.1",
+      "2.0.0"      => "2.0.0",
+      "2.1.2"      => "2.1.0",
+      RUBY_VERSION => RbConfig::CONFIG["ruby_version"]
     }
+
+    platforms = [
+      "i386-mingw32",
+      "universal-known",
+      "universal-unknown",
+      "x64-mingw32",
+      RUBY_PLATFORM
+    ]
+
+    @mock_config_yml = {}
+
+    platforms.collect do |platform|
+      versions.each do |version, api_version|
+        @mock_config_yml["rbconfig-#{platform}-#{version}"] = "/rubies/#{api_version}/rbconfig.rb"
+      end
+    end
+
+    @mock_config_yml
   end
 
   def mock_fake_rb
